@@ -1,43 +1,9 @@
 use super::Pure;
-use crate::compress::util::{always_terminates, is_fine_for_if_cons, negate};
+use crate::compress::util::{is_fine_for_if_cons, negate};
 use swc_common::{util::take::Take, DUMMY_SP};
 use swc_ecma_ast::*;
 
 impl<M> Pure<'_, M> {
-    pub(super) fn drop_unreachable_stmts(&mut self, stmts: &mut Vec<Stmt>) {
-        if !self.options.dead_code && !self.options.side_effects {
-            return;
-        }
-
-        let idx = stmts
-            .iter()
-            .enumerate()
-            .find(|(_, stmt)| always_terminates(stmt));
-
-        if let Some((idx, _)) = idx {
-            stmts.iter_mut().skip(idx + 1).for_each(|stmt| match stmt {
-                Stmt::Decl(
-                    Decl::Var(VarDecl {
-                        kind: VarDeclKind::Var,
-                        ..
-                    })
-                    | Decl::Fn(..),
-                ) => {
-                    // Preserve
-                }
-
-                Stmt::Empty(..) => {
-                    // noop
-                }
-
-                _ => {
-                    self.changed = true;
-                    stmt.take();
-                }
-            });
-        }
-    }
-
     /// # Input
     ///
     /// ```js
