@@ -620,3 +620,95 @@ where
         },
     });
 }
+
+#[derive(Debug, Default)]
+pub(super) struct UnreachableHandler {
+    vars: Vec<Ident>,
+    in_var_name: bool,
+    in_hoisted_var: bool,
+}
+
+impl UnreachableHandler {
+    /// Assumes `s` is not reachable, and preserves variable declarations and
+    /// function declarations in `s`.
+    ///
+    /// Returns true if statement is changed.
+    pub fn preserve_vars(s: &mut Stmt) -> bool {
+        if s.is_empty() {
+            return false;
+        }
+        if let Stmt::Decl(Decl::Var(v)) = s {
+            let mut changed = false;
+            for decl in &mut v.decls {
+                if decl.init.is_some() {
+                    decl.init = None;
+                    changed = true;
+                }
+            }
+
+            return changed;
+        }
+
+        let mut v = Self::default();
+        s.visit_mut_with(&mut v);
+        if v.vars.is_empty() {
+            *s = Stmt::Empty(EmptyStmt { span: DUMMY_SP });
+        } else {
+            *s = Stmt::Decl(Decl::Var(VarDecl {
+                span: DUMMY_SP,
+                kind: VarDeclKind::Var,
+                declare: false,
+                decls: v
+                    .vars
+                    .into_iter()
+                    .map(BindingIdent::from)
+                    .map(Pat::Ident)
+                    .map(|name| VarDeclarator {
+                        span: DUMMY_SP,
+                        name,
+                        init: None,
+                        definite: false,
+                    })
+                    .collect(),
+            }))
+        }
+
+        true
+    }
+}
+
+impl VisitMut for UnreachableHandler {
+    noop_visit_mut_type!();
+
+    fn visit_mut_arrow_expr(&mut self, _: &mut ArrowExpr) {}
+
+    fn visit_mut_fn_decl(&mut self, n: &mut FnDecl) {
+        self.vars.push(n.ident.clone());
+
+        n.function.visit_mut_with(self);
+    }
+
+    fn visit_mut_function(&mut self, _: &mut Function) {}
+
+    fn visit_mut_pat(&mut self, n: &mut Pat) {
+        n.visit_mut_children_with(self);
+
+        if self.in_var_name && self.in_hoisted_var {
+            if let Pat::Ident(i) = n {
+                self.vars.push(i.id.clone());
+            }
+        }
+    }
+
+    fn visit_mut_var_decl(&mut self, n: &mut VarDecl) {
+        self.in_hoisted_var = n.kind == VarDeclKind::Var;
+        n.visit_mut_children_with(self);
+    }
+
+    fn visit_mut_var_declarator(&mut self, n: &mut VarDeclarator) {
+        self.in_var_name = true;
+        n.name.visit_mut_with(self);
+        self.in_var_name = false;
+        n.init.visit_mut_with(self);
+    }
+}
